// MEX wrapper for the sparse_rank_one function in sparsesvd.c
// function [X,U,u,F,k] = sparse_rank_one_mex(A,rho,tol,MaxIter,Info,[F],[X0],[k0]) 

// Last Modified: A. d'Aspremont, Laurent El Ghaoui, Ronny Luss, November 2006.
// http://www.carva.org/alexandre.daspremont


#include "LinfProj.h" 

double *cmpvec;

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	double *x0,*x,lambda,buf;
	int n,i,*idx,idk;
	extern double *cmpvec;
	
	// Inputs:
	x0=mxGetPr(prhs[0]);
	n=(int)mxGetM(prhs[0]);
	lambda= mxGetScalar(prhs[1]);
	// Outputs:
	plhs[0] = mxCreateDoubleMatrix(n, 1, mxREAL);
	x=mxGetPr(plhs[0]);
	// Allocate
	idx=(int*)calloc(n,sizeof(int));
	cmpvec=(double *) calloc(n+1,sizeof(double));
	
	// Iterate
	for (i=0;i<n;i++){x[i]=dabsf(x0[i]);idx[i]=i;} // Initialize idx!!
	modsort(x,idx,n);
	x[idx[n-1]]=x[idx[n-1]]-0.5*lambda;
	idk=1;buf=x[idx[n-1]];
	for (i=1;i<n;i++){
		if (x[idx[n-1-i]]>=(i*x[idx[n-i]]+x[idx[n-1-i]])/(i+1.0)) {
			idk=i+1;
			buf=(i*x[idx[n-i]]+x[idx[n-1-i]])/(i+1.0);
		}
		x[idx[n-1-i]]=(i*x[idx[n-i]]+x[idx[n-1-i]])/(i+1.0);
	}
	if (x[idx[0]]<0){
		for(i=0;i<n;i++){x[i]=0;}}
	else{
		for (i=0;i<idk;i++){
			if (x0[idx[n-1-i]]>=0) {x[idx[n-1-i]]=buf;}
			else {x[idx[n-1-i]]=-buf;}
		}
	}
	for (i=idk;i<n;i++){x[idx[n-1-i]]=x0[idx[n-1-i]];}	
	// Free everything
	free(idx);free(cmpvec);//free(x);
}

// Sorting function, returns indices and sorted vector
void modsort(double *numbers, int *index, int array_size)
{
	int incx=1;
	extern double *cmpvec;
	
	cblas_dcopy(array_size,numbers,incx,cmpvec,incx);
	qsort(index,(size_t) array_size, sizeof(int),cmp);
}

int cmp(const void *keyvali, const void *datumi)
{
	extern double *cmpvec;
	int* keyval = (int*) keyvali;
	int* datum = (int*) datumi;
	
	if (cmpvec[*keyval]==cmpvec[*datum])
	{
		return 0;
	}
	else
	{
		if (cmpvec[*keyval]>cmpvec[*datum]){return 1;}
		else {return -1;}
	}
	
}

double dabsf(double xx)
{
	if (xx>=0)
		return xx;
	else
		return -xx;
}

