function [Y,G,primalvls,gaphist,cpuhist] = SmoothNullPropEUC(A,kv,alphab,tol,maxiter,infov)
% problem size
[m,n]=size(A);gaphist=[];
P=null(A);nk=size(P,2);
% set algo parameters
sigma=4;
mu=tol/(2*log(n+nk));
accel=0.01;
L= accel*8*log(n+nk)/tol; % Lipschitz constant
Nmax = maxiter; 
% check convergence every Nperiod iterations (1% of the time) 
Nperiod=floor(maxiter/infov); 
X = zeros(4*n,n);  % X is the primal variable
Y = zeros(4*n,n);  
F = zeros(4*n,n); % we will store a gradient average here 
disp('--- Smooth Opt. for Null Space Prop. ---')
% main loop
tic;cpuhist=[];dualvls=[];primalvls=[];
for kit=0:Nmax, 
    % eigenvalue decomposition on cone
    U1=X(1:n,1:n);U2=X(n+1:2*n,1:n);U3=X(2*n+1:3*n,1:n);U4=X(3*n+1:4*n,1:n);
    % Compute gradient at new X
    M=[P'*U1*P,-0.5*P'-0.5*P'*U4;-0.5*P-0.5*U4'*P,U2+U3];
    M=0.5*(M+M');fmu=min(eig(M));
    EM=expm((-M+fmu*eye(n+nk))/mu);EM=(EM+EM')/2;
    adjfmu=-(mu*log(trace(EM))-fmu);
    EM=EM/trace(EM); % Smoothing
    G=-[P*EM(1:nk,1:nk)*P';EM(nk+1:nk+n,nk+1:nk+n);EM(nk+1:nk+n,nk+1:nk+n);-0.5*P*EM(1:nk,nk+1:nk+n)];
    % find a projection of Y-Gmu/L on Q1 
    P1=Y-(G/L);
    [xp,yp,zp,wp]=ProjSumBall(vec(P1(1:n,1:n)),vec(P1(n+1:2*n,1:n)),vec(P1(2*n+1:3*n,1:n)),vec(P1(3*n+1:4*n,1:n)),alphab,kv);
    Y=[reshape(xp,n,n);reshape(yp,n,n);reshape(zp,n,n);reshape(wp,n,n)];
    %norm(xp,'inf')+kv^2*norm(yp,'inf')+norm(zp,1)
    % update gradient's weighted average 
    F=F+((kit+1)/2)*G; 
    % project again 
    P2=-(sigma/L)*F; 
    [xp,yp,zp,wp]=ProjSumBall(vec(P2(1:n,1:n)),vec(P2(n+1:2*n,1:n)),vec(P2(2*n+1:3*n,1:n)),vec(P1(3*n+1:4*n,1:n)),alphab,kv);
    Z=[reshape(xp,n,n);reshape(yp,n,n);reshape(zp,n,n);reshape(wp,n,n)]; 
    % update x 
    lambda=2/(kit+3); 
    X = lambda*Z + (1-lambda)*Y;
    cpuhist=[cpuhist;toc];
    primalvls=[primalvls;fmu];
    % check convergence of objective only periodically 
    if (fmu>0)
        disp(['Iter: ',num2str(kit,'%.4e'),'    Gap: ',num2str(gapk,'%.4e'),'    Primal: ',num2str(adjfmu,'%.4e'), '    Dual: ',num2str(dualv,'%.4e'), '    mineig.: ',num2str(fmu,'%.4e')]);
        break;
    end
    if ceil(kit/Nperiod) == fix(kit/Nperiod)
        PGP=P*EM(1:nk,1:nk)*P';
        g1=norm(PGP(:),1);g2=norm(vec(EM(nk+1:nk+n,nk+1:nk+n)/kv^2),1);g3=norm(vec(EM(nk+1:nk+n,nk+1:nk+n)),'inf');g4=norm(vec(EM(nk+1:nk+n,1:nk)'*P)/kv,1);
        dualv=alphab*max([g1,g2,g3,g4])-trace(EM(nk+1:nk+n,1:nk)'*P);dualvls=[dualvls;dualv];
        gapk=min(dualvls)-max(primalvls);gaphist=[gaphist;kit,gapk];
        disp(['Iter: ',num2str(kit,'%.4e'),'    Gap: ',num2str(gapk,'%.4e'),'    Primal: ',num2str(adjfmu,'%.4e'), '    Dual: ',num2str(dualv,'%.4e'), '    mineig.: ',num2str(fmu,'%.4e')]);
        %if gapk <= tol, break, end % gap is non-negative 
    end 
    if kit >= maxiter, break, end % maxiter reached
end 
