% Test semidefinite relaxation for nullspace property
addpath('./JNcode');

% Types:
% 1    strF='Fourier';
% 3    str1='+1/-1';
% 4    strG='Gauss';
% 9    strV='DeVore';

randn('state',100);
n=15;nsamp=2;krange=1:2;rhorange=[0.6,0.7];typelist=[1*ones(1,nsamp),3*ones(1,nsamp),4*ones(1,nsamp)];
%n=40;nsamp=10;krange=1:5;rhorange=[0.5,0.6,0.7,0.8];typelist=[1*ones(1,nsamp),3*ones(1,nsamp),4*ones(1,nsamp)];

if 1
res=[];count=0;
% Iterate over many samples
for rho=rhorange
    for kv=krange
        for tp=typelist
            m = round(rho*n);
            % Generate matrix
            [rss,pnt]=AAGenerateSensingMatrix(tp,n,m);
            A=rss.A;n=rss.n;m=rss.m;
            P=null(A);nk=size(P,2);
            % Test SDP relaxation
            %[alphaSDP,M]=BoundSDP3(P,kv); % Using SDPT3
            %X=P*(M{1}(1:nk,1:nk))*P';Y=M{1}(nk+1:nk+n,nk+1:nk+n);Z=P*(M{1}(1:nk,nk+1:nk+n));
            [alphaSDP,X,Y,Z]=BoundCVX(P,kv); % Using CVX
            % Test LP relaxation
            %alphaLP=BoundJN(A,kv);
            alphaLP=BoundJNmosek(A,kv);
            % Get upper bound on k
            uk=UpperBoundJN(A,kv);
            % Get lower bound on alpha by randomization
            lb=RandzdSDP([X,Z;Z',Y],kv,500000);
            % Display results
            resbuf=[tp,rho,kv,alphaLP,alphaSDP,uk,sum(diag(X).^(1/2)),sum(diag(Y).^(1/2)),max(diag(Y).^(1/2)),lb];
            disp('**************************************************************************************************');
            disp(sprintf('Results: done %d out of %d...',count,length(krange)*length(rhorange)*length(typelist)));
            disp(num2str(resbuf));
            disp('**************************************************************************************************');
            res=[res;resbuf];count=count+1;
        end
    end
end
res
im1=find(res(:,5)>1);
res(im1,5)=ones(length(im1),1);
end

if 1
    clk=clock;
    datestmp=strcat([date,'-',num2str(clk(4)),'h',num2str(clk(5))])
    save(strcat('StatsCompare-',datestmp))
end


% compute aggregate stats %
if 0
    types=[1,3,4];
    % Count number of optimal recovery bounds
    LPopt=(res(:,3)==res(:,6)).*(res(:,4)<.5);
    SDPopt=(res(:,3)==res(:,6)).*(res(:,5)<.5);
    res=[res,LPopt,SDPopt];
    % Compile results
    agres=[];stdres=[];medres=[];minres=[];maxres=[];
    for rho=rhorange
        for kv=krange
            for tp=types
                agres=[agres;mean(res(find((res(:,1)==tp).*(res(:,2)==rho).*(res(:,3)==kv)),:),1)];
                medres=[medres;median(res(find((res(:,1)==tp).*(res(:,2)==rho).*(res(:,3)==kv)),:),1)];
                stdres=[stdres;std(res(find((res(:,1)==tp).*(res(:,2)==rho).*(res(:,3)==kv)),:),0,1)];
                minres=[minres;min(res(find((res(:,1)==tp).*(res(:,2)==rho).*(res(:,3)==kv)),:),[],1)];
                maxres=[maxres;max(res(find((res(:,1)==tp).*(res(:,2)==rho).*(res(:,3)==kv)),:),[],1)];
            end
        end
    end
end

% Produce results table
if 0
    dtab=[];
    typ=3;
    tres=medres(find(medres(:,1)==typ),:);
    atres=agres(find(agres(:,1)==typ),:);
    for rho=rhorange
        bufLP=tres(find(tres(:,2)==rho),4)';bufSDP=tres(find(tres(:,2)==rho),5)';buflow=tres(find(tres(:,2)==rho),10)';
        dtab=[dtab;[rho,bufLP;rho,bufSDP;rho,buflow]];
        min(tres(find(tres(:,2)==rho),6)')
    end
    latex(dtab,'nomath','%.2f');
end


% Plot alpha_s as a function of s
if 0
    typ=4;rho=.7;
    exi=find((medres(:,1)==tp).*(medres(:,2)==rho));
%    plot(krange,minres(exi,4),':b','LineWidth',2,'MarkerSize',35);hold on;
%    plot(krange,maxres(exi,4),':b','LineWidth',2,'MarkerSize',35);
    plot(krange,minres(exi,5),':k','LineWidth',2,'MarkerSize',35);hold on;
    plot(krange,maxres(exi,5),':k','LineWidth',2,'MarkerSize',35);
    plot(krange,medres(exi,5),'-k.','LineWidth',4,'MarkerSize',35);    
%    plot(krange,medres(exi,4),'-b.','LineWidth',4,'MarkerSize',35);
    hold off;
    set(gca,'FontSize',16);xlabel('k');ylabel('alphak');
    text(3,.2,'l1');text(2,.75,'l0');
    line([min(krange) max(krange)],[1 1],'LineWidth',3)
    line([min(krange) max(krange)],[1/2 1/2],'LineWidth',3,'LineStyle','--')
    axis([min(krange) max(krange) 0 1.1])
end

% Histogram of sum(diag(X).^(1/2))
if 0
    delta=3;
    not1=find(res(:,3)>1);
    valsa=(sqrt(2/pi)+sqrt(2*log(delta)))*res(not1,7);
    valsb=max([((sqrt(2/pi)+sqrt(2*log(delta)))*res(not1,8)./res(not1,3))';...
        ((sqrt(2*log(2*n))+sqrt(2*log(delta)))*res(not1,9))'])';
    [n0,xout] = hist(valsa.*valsb,30); %hist(res(not1,7),30);
    h=bar(xout,n0,'histc');xlabel('XY12');ylabel('numoccur');
    set(h,'FaceColor',[0.8,0.8,0.8],'EdgeColor',[0.3,0.3,0.3]);
    set(gca,'FontSize',16);
    figure;
    plot(res(:,3),res(:,7),'bo','MarkerSize',12);
    set(gca,'FontSize',16);
    axis([0 max(krange)+1 0 max(res(:,7))+0.2]);
    figure;
    idx=find((res(:,3)==1).*(res(:,1)~=1));
    valsa11=(sqrt(2/pi)+sqrt(2*log(delta)))*res(idx,7);
    valsb11=max([((sqrt(2/pi)+sqrt(2*log(delta)))*res(idx,8)./res(idx,3))';...
        ((sqrt(2*log(2*n))+sqrt(2*log(delta)))*res(idx,9))'])';
    [n1,yout1] = hist(valsa11.*valsb11,2);%hist(res(idx,7),13);
    h=bar(yout1,n1,'histc');xlabel('1XY12');ylabel('numoccur');hold on;
    set(h,'FaceColor',[0.8,0.8,0.8],'EdgeColor',[0.3,0.3,0.3])
    set(gca,'FontSize',16);
    idx=find((res(:,3)==1).*(res(:,1)==1));
    valsa12=(sqrt(2/pi)+sqrt(2*log(delta)))*res(idx,7);
    valsb12=max([((sqrt(2/pi)+sqrt(2*log(delta)))*res(idx,8)./res(idx,3))';...
        ((sqrt(2*log(2*n))+sqrt(2*log(delta)))*res(idx,9))'])';
    [n2,yout2] = hist(valsa12.*valsb12,30);%hist(res(idx,7),30);
    h=bar(yout2,n2,'histc');xlabel('1XY12');ylabel('numoccur');
    set(h,'FaceColor',[0.5,0.5,0.5],'EdgeColor',[0.3,0.3,0.3])
    axis([1 max(yout2)+5 0 max(n1)+2]);
    set(gca,'FontSize',16);hold off;
end



